@@ -171,6 +171,60 @@ def test_build_already_encoded():
     assert str(u) == "http://историк.рф/путь/файл?ключ=знач#фраг"
 
 
+def test_build_percent_encoded():
+    u = URL.build(
+        scheme="http",
+        host="%2d.org",
+        user="u%2d",
+        password="p%2d",
+        path="/%2d",
+        query_string="k%2d=v%2d",
+        fragment="f%2d",
+    )
+    assert str(u) == "http://u%252d:p%252d@%2d.org/%252d?k%252d=v%252d#f%252d"
+    assert u.raw_host == "%2d.org"
+    assert u.host == "%2d.org"
+    assert u.raw_user == "u%252d"
+    assert u.user == "u%2d"
+    assert u.raw_password == "p%252d"
+    assert u.password == "p%2d"
+    assert u.raw_authority == "u%252d:p%252d@%2d.org"
+    assert u.authority == "u%2d:p%2d@%2d.org:80"
+    assert u.raw_path == "/%252d"
+    assert u.path == "/%2d"
+    assert u.query == {"k%2d": "v%2d"}
+    assert u.raw_query_string == "k%252d=v%252d"
+    assert u.query_string == "k%2d=v%2d"
+    assert u.raw_fragment == "f%252d"
+    assert u.fragment == "f%2d"
+
+
+def test_build_with_authority_percent_encoded():
+    u = URL.build(scheme="http", authority="u%2d:p%2d@%2d.org")
+    assert str(u) == "http://u%252d:p%252d@%2d.org"
+    assert u.raw_host == "%2d.org"
+    assert u.host == "%2d.org"
+    assert u.raw_user == "u%252d"
+    assert u.user == "u%2d"
+    assert u.raw_password == "p%252d"
+    assert u.password == "p%2d"
+    assert u.raw_authority == "u%252d:p%252d@%2d.org"
+    assert u.authority == "u%2d:p%2d@%2d.org:80"
+
+
+def test_build_with_authority_percent_encoded_already_encoded():
+    u = URL.build(scheme="http", authority="u%2d:p%2d@%2d.org", encoded=True)
+    assert str(u) == "http://u%2d:p%2d@%2d.org"
+    assert u.raw_host == "%2d.org"
+    assert u.host == "%2d.org"
+    assert u.user == "u-"
+    assert u.raw_user == "u%2d"
+    assert u.password == "p-"
+    assert u.raw_password == "p%2d"
+    assert u.authority == "u-:p-@%2d.org:80"
+    assert u.raw_authority == "u%2d:p%2d@%2d.org"
+
+
 def test_build_with_authority_with_path_with_leading_slash():
     u = URL.build(scheme="http", host="example.com", path="/path_with_leading_slash")
     assert str(u) == "http://example.com/path_with_leading_slash"
